﻿using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace FloorMod
    {
    //The ModBehaviour class inherits from MonoBehaviour and can be used as such
    //Each class that inherits from ModBehavior will have a copy instantiated when the mod is first loaded
    //All ModBehaviours are enabled and disabled based on whether the mod has been activated by the player
    public class FloorBehaviour : ModBehaviour
        {
        private int _maxFloor = 20;
        public int MaxFloor
            {
            get { return _maxFloor; }
            set
                {
                _maxFloor = value;
                //The active state of the MonoBehavior will reflect whether the Mod is activated
                //Or put another way isActiveAndEnabled == ParentMod.Active
                if (isActiveAndEnabled)
                    {
                    GameSettings.MaxFloor = _maxFloor;
                    }
                }
            }

		//An inputfield on top of the floor label to allow player to change floor directly
        public InputField FloorChanger;

        //This is a MonoBehavior method, you can look up all callbacks in the Unity documentation
        void Start()
            {
            //All ModBehaviours has a function to load settings from the mod's global settings file
            //Note that everything is saved in strings
            //This function uses the default string converter for the generic type argument, e.g. a 32 bit integer here
            MaxFloor = LoadSetting("MaxFloor", 20);
            //This is a unity callback for when a new scene is loaded
            SceneManager.sceneLoaded += OnLevelFinishedLoading;
            }

        private void OnLevelFinishedLoading(Scene scene, LoadSceneMode mode)
            {
            //Everytime we hit the main menu we want to reset the maxfloor value to the default global value
            //Other scenes include MainScene and Customization
            if (scene.name.Equals("MainMenu"))
                {
                MaxFloor = LoadSetting("MaxFloor", 20);
                }
            if (scene.name.Equals("MainScene"))
                {
                if (isActiveAndEnabled)
                    {
                    CreateFloorChanger();
                    }
                }
            }
		
        private void CreateFloorChanger()
            {
			//First find the floor label, with output from the UI_UNDER_MOUSE terminal command
            var floorLabel = WindowManager.FindElementPath("MainPanel/Holder/FloorLabelPanel");
            if (floorLabel != null)
                {
                FloorChanger = WindowManager.SpawnInputbox();
				//Change the background color of the inputfield to not be transparent
                var selectable = FloorChanger.GetComponent<UnityEngine.UI.Selectable>();
                var colors = selectable.colors;
                colors.normalColor = new Color(0.8f, 0.8f, 0.8f);
                selectable.colors = colors;
				//Remove the placeholder text[0] and put the actual text[1] in the center of the inputfield
                var texts = FloorChanger.GetComponentsInChildren<Text>();
                texts[0].text = "";
                texts[1].fontSize = 12;
                texts[1].alignment = TextAnchor.MiddleCenter;
				//Change the text to not have any margins
                WindowManager.AddElementToElement(texts[1].gameObject, FloorChanger.gameObject, new Rect(0, 0, 0, 0),
                    new Rect(0, 0, 1, 1));
				//Add the inputfield on top of the floor label with 0 margins
                WindowManager.AddElementToElement(FloorChanger.gameObject, floorLabel.gameObject, new Rect(0, 0, 0, 0), new Rect(0, 0, 1, 1));
                FloorChanger.onEndEdit.AddListener(OnFloorChanged);
                }
            }

        private static bool _disableUpdate = false;
        private void OnFloorChanged(string value)
            {
			//We don't want to update the floor if we changed the value in code
            if (!_disableUpdate)
                {
                var floor = Mathf.Clamp(value.ConvertToIntDef(GameSettings.Instance.ActiveFloor), -1, GameSettings.MaxFloor);
				//This function does abunch of stuff to make sure the floor change is valid
                CameraScript.Instance.ChangeFloor(floor - GameSettings.Instance.ActiveFloor);
                _disableUpdate = true;
                FloorChanger.text = GameSettings.Instance.ActiveFloor.ToString();
                _disableUpdate = false;
                }
            }

        void Update()
            {
			//Update the inputfield value every frame to the current floor
            if (FloorChanger != null && !FloorChanger.isFocused && GameSettings.Instance != null)
                {
                _disableUpdate = true;
                FloorChanger.text = GameSettings.Instance.ActiveFloor.ToString();
                _disableUpdate = false;
                }
            }

        //Mods are always deactivated by default, wait for OnActivate to be called before doing stuff
        //This method will ALWAYS be called when the game launches if this mod is activated
        //and everytime the mod is toggled on by the user
        public override void OnActivate()
            {
            GameSettings.MaxFloor = MaxFloor;
            CreateFloorChanger();
            }

        //Use this method to clean up your mod
        //The MonoBehaviour is disabled immediatly after this method has been called
        public override void OnDeactivate()
            {
            GameSettings.MaxFloor = 10;
            if (FloorChanger != null)
                {
                Destroy(FloorChanger.gameObject);
                }
            }
        }
    }
