﻿using System;
using System.Linq;
using UnityEngine;

namespace FloorMod
    {
    //Your mod should have exactly one class that implements the abstract ModMeta class
    public class Main : ModMeta
        {
        private FloorBehaviour _floorBehaviour;

        //This method is called once when the mod is first loaded
        public override void Initialize(ModController.DLLMod parentMod)
            {
            //We need a reference to the FloorBehaviour to read and write from the mod settings file
            _floorBehaviour = parentMod.Behaviors.OfType<FloorBehaviour>().First();
            }

        //This function is used to generate the content in the "Mods" section of the options window
        //The parent RectTransform is the panel that should be populated with content
        //The inGame variable tells you whether we are in an active game as opposed to the main menu
        public override void ConstructOptionsScreen(RectTransform parent, bool inGame)
            {
            //Start by spawning a label
            var label = WindowManager.SpawnLabel();
            label.text = "Maximum floors:";
            //Add the label to the mod panel at (0, 0) with 96 width and 32 height, anchored to the top left
            WindowManager.AddElementToElement(label.gameObject, parent.gameObject, new Rect(0, 0, 96, 32),
                new Rect(0, 0, 0, 0));
            //Add an inputfield where the user can write text to change the maximum floor level
            var inputField = WindowManager.SpawnInputbox();
            inputField.text = _floorBehaviour.MaxFloor.ToString();
            inputField.onValueChanged.AddListener(x =>
                {
                    var newValue = Mathf.Max(x.ConvertToIntDef(10), 1);
                    _floorBehaviour.MaxFloor = newValue;
                    //If we are in the main menu, we want to save the user input as default
                    //Otherwise we save it when the game is saved
                    if (!inGame)
                        {
                        //Settings are saved to a text file with the dll, all ModBehaviours has a function to do this
                        _floorBehaviour.SaveSetting("MaxFloor", newValue.ToString());
                        }
                });
            WindowManager.AddElementToElement(inputField.gameObject, parent.gameObject, new Rect(100, 0, 96, 32),
                new Rect(0, 0, 0, 0));
            }

        //This function is called when the player saves their game, assuming the mod is activated
        //If you do not override this function, or return null, nothing will be saved to file
        public override WriteDictionary Serialize(GameReader.LoadMode mode)
            {
            //The WriteDictionary is a simple dictionary the game uses to save data to a file
            //Note that the type saved needs to be a value type or have an empty contructor
            //You can safely save arrays, lists, hashsets and dictionaries
            //In this case we are saving a 32 bit integer, which is a value type
            var data = new WriteDictionary();
            data["Floor"] = _floorBehaviour.MaxFloor;
            return data;
            }

        //This file is called when a save game is loaded, if there is any data relevant to this ACTIVE mod
        //Note that this function is called very early in the loading process, so all save game data might not be initialized yet
        public override void Deserialize(WriteDictionary data, GameReader.LoadMode mode)
            {
            //The WriteDictionary Get function tries to fetch a value from the dictionary
            //but will return a default value if the key is not present
            //In this case MaxFloor is returned if the "Floor" key is not present
            _floorBehaviour.MaxFloor = data.Get("Floor", _floorBehaviour.MaxFloor);
            //Note that we are not saving this value to the settings file, as this is save file specific
            //This way the player can have a default value for all new games, and a value for specific saves
            }

        //Note that this name is used to identify the mod when saving and loading game data
        public override string Name
            {
            get { return "All the floors!"; }
            }
        }
    }